# This file was auto-generated by salt's setup on Friday, 07 October 2016 @ 17:10:41 UTC.

from salt.version import SaltStackVersion

__saltstack_version__ = SaltStackVersion(2016, 11, 0, 0, u'rc', 1, 0, None)
